#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>
#include <cmath>
#include <time.h>
#include "../lib/orga2.h" 
//The OpenCL C++ bindings, with exceptions
#define __CL_ENABLE_EXCEPTIONS
#include <CL/cl.hpp>

using namespace std;

void deviceInfo(cl::Device &d){
	cout << "\t\tType: \t" << d.getInfo<CL_DEVICE_TYPE>() << endl;
    cout << "\t\tVendor ID: \t" << d.getInfo<CL_DEVICE_VENDOR_ID>() << endl;
    cout << "\t\tMax. Compute Units: \t" << d.getInfo<CL_DEVICE_MAX_COMPUTE_UNITS>() << endl;
    cout << "\t\tMax. WorkItem Dimensions: \t" << d.getInfo<CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS>() << endl;
    cout << "\t\tPreferred INT vector width: \t" << d.getInfo<CL_DEVICE_PREFERRED_VECTOR_WIDTH_INT>() << endl;
    cout << "\t\tPreferred LONG vector width: \t" << d.getInfo<CL_DEVICE_PREFERRED_VECTOR_WIDTH_LONG>() << endl;
    cout << "\t\tPreferred FLOAT vector width: \t" << d.getInfo<CL_DEVICE_PREFERRED_VECTOR_WIDTH_FLOAT>() << endl;
    cout << "\t\tPreferred DOUBLE vector width: \t" << d.getInfo<CL_DEVICE_PREFERRED_VECTOR_WIDTH_DOUBLE>() << endl;
    cout << "\t\tMax. Clock frec: \t" << d.getInfo<CL_DEVICE_MAX_CLOCK_FREQUENCY>() << endl;
    cout << "\t\tMax. Mem alloc size: \t" << d.getInfo<CL_DEVICE_MAX_MEM_ALLOC_SIZE>() << endl;
    cout << "\t\tGlobal Cacheline size: \t" << d.getInfo<CL_DEVICE_GLOBAL_MEM_CACHELINE_SIZE>() << endl;
    cout << "\t\tGlobal Cache size: \t" << d.getInfo<CL_DEVICE_GLOBAL_MEM_CACHE_SIZE>() << endl;
    cout << "\t\tGlobal Mem. size: \t" << d.getInfo<CL_DEVICE_GLOBAL_MEM_SIZE>() << endl;
    cout << "\t\tLocal Mem. size: \t" << d.getInfo<CL_DEVICE_LOCAL_MEM_SIZE>() << endl;
    cout << "\t\tProfiling timer resolution: \t" << d.getInfo<CL_DEVICE_PROFILING_TIMER_RESOLUTION>() << endl;
    cout << "\t\tLittle Endian: \t" << d.getInfo<CL_DEVICE_ENDIAN_LITTLE>() << endl;
    cout << "\t\tAvailable: \t" << d.getInfo<CL_DEVICE_AVAILABLE>() << endl;
    cout << "\t\tName: \t" << d.getInfo<CL_DEVICE_NAME>() << endl;
    cout << "\t\tVendor: \t" << d.getInfo<CL_DEVICE_VENDOR>() << endl;
    cout << "\t\tVersion: \t" << d.getInfo<CL_DRIVER_VERSION>() << endl;
    cout << "\t\tProfiles: \t" << d.getInfo<CL_DEVICE_PROFILE>() << endl;
    cout << "\t\tVersion: \t" << d.getInfo<CL_DEVICE_VERSION>() << endl;
    cout << "\t\tExtensions: \t" << d.getInfo<CL_DEVICE_EXTENSIONS>() << endl;
}

void platformInfo(cl::Platform &p){
	unsigned int i;
	cout << "Platform Information: \n";
	cout << "\tProfile: \t" 	<< p.getInfo<CL_PLATFORM_PROFILE>() 	<< endl;
	cout << "\tName: \t\t" 		<< p.getInfo<CL_PLATFORM_NAME>() 		<< endl;
	cout << "\tVersion: \t" 	<< p.getInfo<CL_PLATFORM_VERSION>() 	<< endl;
	cout << "\tVendor: \t" 		<< p.getInfo<CL_PLATFORM_VENDOR>() 		<< endl;
	cout << "\tExtensions: \t" 	<< p.getInfo<CL_PLATFORM_EXTENSIONS>() 	<< endl;
	vector<cl::Device> devices;
	p.getDevices(CL_DEVICE_TYPE_ALL, &devices);
	forn(i, devices.size()){
		cout << "\tDevice number: " << i << endl;
		deviceInfo(devices[i]);
	}	
}

int main(){
	try{
		unsigned int i;
		vector<cl::Platform> platforms;
		cl::Platform::get(&platforms);
		forn(i, platforms.size()) {
			platformInfo(platforms[i]);
		}
	} catch (cl::Error& err){
	  clErr(err);
	  return -1;
	}
}
