#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>
#include <cmath>
#include "../lib/orga2.h" 
//The OpenCL C++ bindings, with exceptions
#define __CL_ENABLE_EXCEPTIONS
#define KERFILE "DotProduct.cl" //source of the kernels
#define KERFUNC "dotProduct" //name of the kernel to be used
#include <CL/cl.hpp>

using namespace std;
bool error = false;

bool isCorrect(const vector<float> &a, const vector<float> &b, const vector<float> &result){
	//check if sum(result) = <a;b>
	unsigned int i;
	float sumRes = 0;
	float dotProd = 0;
	forn(i, result.size()) sumRes += result[i];
	return true;
	forn(i, a.size()) dotProd += a[i]*b[i];
	return abs(sumRes - dotProd)/abs(dotProd) < NUMERICAL_ERROR;
}

void check(const vector<float> &a, const vector<float> &b, const vector<float> result){
	if(!isCorrect(a,b,result)){
		error = true;
		std::cout << "NUMERICAL_ERROR = " << NUMERICAL_ERROR << std::endl; 
		std::cout << "Test failed on inputs: " << std::endl << std::endl;
		printVector(a);
		std::cout << std::endl; 
		printVector(b);
		
		std::cout << std::endl;
	}
}

int main(int argc, char** argv){
	unsigned int n;
	try{
		timerColNames(CPU);
		for(n = VECTOR_STEP; n <= VECTOR_MAX_SIZE; n += VECTOR_STEP){
			std::cout << n << "\t";
			unsigned int i;
			int work_groups = oneDimSize(n)/WORK_GROUP_SIZE;
			//std::cout << " WG = " << work_groups << std::endl;

			vector<float> v_inputA(n,0);
			vector<float> v_inputB(n,0);
			vector<float> output(work_groups, 0);
			unsigned int k;
			float tempRes;
			timer(INIT);
			forn(i, NUM_TESTS){
				//create some data
				randomizeFloatVector(v_inputA);
				randomizeFloatVector(v_inputB);

				timer(TOGGLE);

				tempRes = 0;
				forn(k, v_inputA.size())  tempRes += v_inputA[k] * v_inputB[k];
				output[0] = tempRes;
				timer(TOGGLE);
				
				#ifdef TEST
				check(v_inputA, v_inputB, output);
				std::cout << std::endl;
				if(!error) std::cout << "Test: Success (" << NUM_TESTS << " inputs)" << std::endl;
				else std::cout << "Test: Failed" << std::endl;
				#endif
			}
			timer(FINISH);
		}
	} catch (cl::Error& err){
	  clErr(err);
	  return -1;
	}
}
