#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>
#include <cmath>
#include "../lib/orga2.h" 
//The OpenCL C++ bindings, with exceptions
#define __CL_ENABLE_EXCEPTIONS
#define KERFILE "DotProduct.cl" //source of the kernels
#define KERFUNC "dotProduct" //name of the kernel to be used
#include <CL/cl.hpp>

using namespace std;
bool error = false;

bool isCorrect(const vector<float> &a, const vector<float> &b, const vector<float> &result){
	//check if sum(result) = <a;b>
	unsigned int i;
	float sumRes = 0;
	float dotProd = 0;
	forn(i, result.size()) sumRes += result[i];
	return true;	//para testear tiempo
	forn(i, a.size()) dotProd += a[i]*b[i];
	return abs(sumRes - dotProd)/abs(dotProd) < NUMERICAL_ERROR;
}

void check(const vector<float> &a, const vector<float> &b, const vector<float> result){
	if(!isCorrect(a,b,result)){
		error = true;
		std::cout << "NUMERICAL_ERROR = " << NUMERICAL_ERROR << std::endl; 
		std::cout << "Test failed on inputs: " << std::endl << std::endl;
		printVector(a);
		std::cout << std::endl; 
		printVector(b);
		
		std::cout << std::endl;
	}
}

int main(int argc, char** argv){
	unsigned int n;
	try{
		timerColNames(GPU);
		for(n = VECTOR_STEP; n <= VECTOR_MAX_SIZE; n += VECTOR_STEP){
			std::cout << n << "\t";
			unsigned int i;
			vector<cl::Platform> platforms;
			cl::Platform::get(&platforms);
			vector<cl::Device> devices;
			platforms.front().getDevices(CL_DEVICE_TYPE_GPU, &devices);
			cl::Context context(devices);
			cl::Device device = devices.front();

			//load kernel from filename
			std::string kernelSource = convertToString(KERFILE);
			cl::Program::Sources sourceCode;
			sourceCode.push_back(std::make_pair(kernelSource.c_str(), kernelSource.size()));
			
			//create a program
			cl::Program program = cl::Program(context, sourceCode);
			
			//build it for all devices
			try{ program.build(devices); }
			catch(cl::Error &err){
				std::cerr << "Error building kernel " << KERFILE << std::endl;
				tr(devices, it){
					std::cerr << "\t" << it->getInfo<CL_DEVICE_NAME>() << " log: " 
					<< program.getBuildInfo<CL_PROGRAM_BUILD_LOG>(*it) << std::endl;
				}
				return -1;
			}
			cl::CommandQueue cmdQueue(context, device);

			int work_groups = oneDimSize(n)/WORK_GROUP_SIZE;
			//std::cout << " WG = " << work_groups << std::endl;

			cl::Buffer inputBufferA(context, CL_MEM_READ_WRITE, n*sizeof(cl_float));
			cl::Buffer inputBufferB(context, CL_MEM_READ_WRITE, n*sizeof(cl_float));
			cl::Buffer outputBuffer(context, CL_MEM_READ_WRITE, work_groups * sizeof(cl_float));
			//initialize kernel
			cl::Kernel kernel(program, KERFUNC);
			cl::KernelFunctor kernel_func = kernel.bind(cmdQueue, cl::NDRange(oneDimSize(n)), cl::NDRange(WORK_GROUP_SIZE));

			vector<float> v_inputA(n,0);
			vector<float> v_inputB(n,0);
			vector<float> output(work_groups, 0);
			timer(INIT);
			forn(i, NUM_TESTS){
				//create some data
				randomizeFloatVector(v_inputA);
				randomizeFloatVector(v_inputB);
				timer(TOGGLE);
				cmdQueue.enqueueWriteBuffer(inputBufferA, true, 0, n*sizeof(cl_float), &v_inputA[0]);	//write buf 2 device main mem
				cmdQueue.enqueueWriteBuffer(inputBufferB, true, 0, n*sizeof(cl_float), &v_inputB[0]);	//write buf 2 device main mem

				//execute
				kernel_func(inputBufferA, inputBufferB, outputBuffer, n, cl::__local(sizeof(cl_float) * WORK_GROUP_SIZE));

				cmdQueue.enqueueReadBuffer(outputBuffer, true, 0, work_groups * sizeof(cl_float), &output[0]);	//read to host mem from dev mem
				timer(TOGGLE);
				#ifdef TEST
				check(v_inputA, v_inputB, output);
				std::cout << std::endl;
				if(!error) std::cout << "Test: Success (" << NUM_TESTS << " inputs)" << std::endl;
				else std::cout << "Test: Failed" << std::endl;
				#endif
			}
			timer(FINISH);
		}
	} catch (cl::Error& err){
	  clErr(err);
	  return -1;
	}
}
