/* A es de nxm y
   B es de mxl. Por lo tanto AB es de nxl.
   Se asumen n,m,l divisibles por BLOCK_SIZE.
   n viene implicito por la cantidad de WorkGroups
*/

#define BLOCK_SIZE 16

__kernel void matrixMul(
	const __global float *A,
	const __global float *B,
	__global float *AB, 
	const unsigned int m,
	const unsigned int l )
{
	float ijAcum = 0;				//acumulador del producto
	int bx = get_group_id(0);	//indice x del bloque
	int by = get_group_id(1);	//indice y del bloque
	int j  = get_local_id(0);	//indice x dentro del bloque
	int i  = get_local_id(1);   //indice y dentro del bloque

	int aBegin = m * BLOCK_SIZE * by;	//aca esta el (0,0) del bloque de A actual
	int aEnd   = aBegin + m - 1; 		//a lo sumo llega hasta aca 
										//(aca hacemos uso de que BLOCK_SIZE divide a m,n,l
	int aStep  = BLOCK_SIZE;			//salto del bloque A, hacer dibujito para entender

	int bBegin = BLOCK_SIZE * bx;	//aca esta el (0,0) del bloque de B actual
	int bStep = BLOCK_SIZE * l;		//salto del bloque B, hacer dibujito para entender (es una L)
	
	for(int a = aBegin, b = bBegin; a <= aEnd; a += aStep, b += bStep){
		//copiamos las submatrices a memoria local, cada W-Item copia su indice
		__local float As[BLOCK_SIZE][BLOCK_SIZE];
		__local float Bs[BLOCK_SIZE][BLOCK_SIZE];
		As[i][j] = A[a + m*i + j];	//hacer dibujito para ver el indice
		Bs[i][j] = B[b + l*i + j];	//idem

		barrier(CLK_LOCAL_MEM_FENCE);	//esperamos a que las submatrices se carguen
		for(int k = 0; k < BLOCK_SIZE; k++) ijAcum += As[i][k] * Bs[k][j];
		barrier(CLK_LOCAL_MEM_FENCE);	//esperamos asi nadie pisa las matrices As y Bs
	}
	int c = l * BLOCK_SIZE * by + BLOCK_SIZE * bx;	//el (0,0) del bloque bx by
	AB[c + l*i+j] = ijAcum;
}
