#define BLOCK_SIZE 16	//debe coincidir con el BLOCK_SIZE del cpp
#define CHANNELS 3

__kernel void medianFilter(
	const __global unsigned char *img,
	__global unsigned char *out, 
	const unsigned int width,
	const unsigned int height,
	const unsigned int step)
{
	
	int ijAcum[9];	
	int j  = get_local_id(0);	//indice x dentro del bloque
	int i  = get_local_id(1);   //indice y dentro del bloque

	int gidj  = get_global_id(0);	//indice x global
	int gidi  = get_global_id(1);   //indice y global

	int s, t;	//para ciclos
	int temp, min;

	if(	gidj < width && 
		gidi < height )//procesamos dentro de la imagen
	{
		__local unsigned char M[(BLOCK_SIZE*BLOCK_SIZE+4*BLOCK_SIZE+4)*CHANNELS];	//el bloque y los bordes
		int stepM = (BLOCK_SIZE+2)*CHANNELS;

		M[(i+1)*stepM+(j+1)*CHANNELS+0] = img[gidi*step+gidj*CHANNELS+0]; //el i+1 hace rol del central, el i del elem de arriba e i+2 del de abajo
		M[(i+1)*stepM+(j+1)*CHANNELS+1] = img[gidi*step+gidj*CHANNELS+1]; 
		M[(i+1)*stepM+(j+1)*CHANNELS+2] = img[gidi*step+gidj*CHANNELS+2]; 
		//copia los bordes
		barrier(CLK_LOCAL_MEM_FENCE);
		if(i == 0 && gidi != 0) {
			M[(i)*stepM+(j+1)*CHANNELS+0] = img[(gidi-1)*step+gidj*CHANNELS+0];
			M[(i)*stepM+(j+1)*CHANNELS+1] = img[(gidi-1)*step+gidj*CHANNELS+1];
			M[(i)*stepM+(j+1)*CHANNELS+2] = img[(gidi-1)*step+gidj*CHANNELS+2];
		}
		if(i == BLOCK_SIZE-1 && gidi != height-1) {
			M[(i+2)*stepM+(j+1)*CHANNELS+0] = img[(gidi+1)*step+gidj*CHANNELS+0];
			M[(i+2)*stepM+(j+1)*CHANNELS+1] = img[(gidi+1)*step+gidj*CHANNELS+1];
			M[(i+2)*stepM+(j+1)*CHANNELS+2] = img[(gidi+1)*step+gidj*CHANNELS+2];
		}
		if(j == 0 && gidj != 0) {
			M[(i+1)*stepM+(j)*CHANNELS+0] = img[(gidi)*step+(gidj-1)*CHANNELS+0];
			M[(i+1)*stepM+(j)*CHANNELS+1] = img[(gidi)*step+(gidj-1)*CHANNELS+1];
			M[(i+1)*stepM+(j)*CHANNELS+2] = img[(gidi)*step+(gidj-1)*CHANNELS+2];
		}
		if(j == BLOCK_SIZE-1 && gidj != width-1) {
			M[(i+1)*stepM+(j+2)*CHANNELS+0] = img[(gidi)*step+(gidj+1)*CHANNELS+0];
			M[(i+1)*stepM+(j+2)*CHANNELS+1] = img[(gidi)*step+(gidj+1)*CHANNELS+1];
			M[(i+1)*stepM+(j+2)*CHANNELS+2] = img[(gidi)*step+(gidj+1)*CHANNELS+2];
		}
		if(i == 0 && j == 0 && gidj != 0 && gidi != 0) {
			M[(i)*stepM+(j)*CHANNELS+0] = img[(gidi-1)*step+(gidj-1)*CHANNELS+0];
			M[(i)*stepM+(j)*CHANNELS+1] = img[(gidi-1)*step+(gidj-1)*CHANNELS+1];
			M[(i)*stepM+(j)*CHANNELS+2] = img[(gidi-1)*step+(gidj-1)*CHANNELS+2];
		}
		if(i == 0 && j == BLOCK_SIZE-1 && gidj != width-1 && gidi != 0) {
			M[(i)*stepM+(j+2)*CHANNELS+0] = img[(gidi-1)*step+(gidj+1)*CHANNELS+0];
			M[(i)*stepM+(j+2)*CHANNELS+1] = img[(gidi-1)*step+(gidj+1)*CHANNELS+1];
			M[(i)*stepM+(j+2)*CHANNELS+2] = img[(gidi-1)*step+(gidj+1)*CHANNELS+2];
		}
		if(i == BLOCK_SIZE-1 && j == BLOCK_SIZE-1 && gidj != width-1 && gidi != height-1) {
			M[(i+2)*stepM+(j+2)*CHANNELS+0] = img[(gidi+1)*step+(gidj+1)*CHANNELS+0];
			M[(i+2)*stepM+(j+2)*CHANNELS+1] = img[(gidi+1)*step+(gidj+1)*CHANNELS+1];
			M[(i+2)*stepM+(j+2)*CHANNELS+2] = img[(gidi+1)*step+(gidj+1)*CHANNELS+2];
		}
		if(i == BLOCK_SIZE-1 && j == 0 && gidj != 0 && gidi != height-1) {
			M[(i+2)*stepM+(j)*CHANNELS+0] = img[(gidi+1)*step+(gidj-1)*CHANNELS+0];
			M[(i+2)*stepM+(j)*CHANNELS+1] = img[(gidi+1)*step+(gidj-1)*CHANNELS+1];
			M[(i+2)*stepM+(j)*CHANNELS+2] = img[(gidi+1)*step+(gidj-1)*CHANNELS+2];
		}
		barrier(CLK_LOCAL_MEM_FENCE);
		for(int k = 0; k < CHANNELS; k++){
			ijAcum[0] = M[(i)*stepM	+ (j)*CHANNELS+k]	;
			ijAcum[1] = M[(i)*stepM	+ (j+1)*CHANNELS+k]	;
			ijAcum[2] = M[(i)*stepM	+ (j+2)*CHANNELS+k]	;
			ijAcum[3] = M[(i+1)*stepM	+ (j)*CHANNELS+k];
			ijAcum[4] = M[(i+1)*stepM	+ (j+1)*CHANNELS+k];
			ijAcum[5] = M[(i+1)*stepM	+ (j+2)*CHANNELS+k];
			ijAcum[6] = M[(i+2)*stepM	+ (j)*CHANNELS+k]  ;
			ijAcum[7] = M[(i+2)*stepM	+ (j+1)*CHANNELS+k];
			ijAcum[8] = M[(i+2)*stepM	+ (j+2)*CHANNELS+k];
			//sort
			for(s = 0; s < 9; s++){
				min = s;
				for(t = s; t < 9; t++){
					if(ijAcum[t] < ijAcum[min]) min = t;
				}
				temp = ijAcum[s];
				ijAcum[s] = ijAcum[min];
				ijAcum[min] = temp;
				barrier(CLK_LOCAL_MEM_FENCE);
			}
			out[gidi*step+gidj*CHANNELS+k] = ijAcum[4];	
		}
	}
}
