#include <stdlib.h>
#include <iostream>
#include <ctime>
#include <cmath>
#include <cv.h>
#include <highgui.h>

#define forsn(i,s,n) for(i=s;i<n;++i)
#define forn(i,n) forsn(i,0,n)

using namespace std;

void cvPrint(char* str, IplImage* img){
	CvFont font;
	double hScale=1.0;
	double vScale=1.0;
	int    lineWidth=2;
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, hScale,vScale,0,lineWidth);
	cvPutText (img,str,cvPoint(40,40), &font, cvScalar(255,255,0));
}


uchar sat(int x){
	if(x<0) return 0;
	else if (x>255) return 255;
	uchar t = x;
	return t;
}

IplImage* convolve_copy(IplImage* src, int *m){	//la matriz m es de 3x3
	int height,width,step,channels;
	uchar *data, *srcd;
	int i,j,k;

	int acum, acumX, acumY;
	// get the image data
	height    = src->height;
	width     = src->width;
	step      = src->widthStep;
	channels  = src->nChannels;
	srcd      = (uchar *)src->imageData;

	IplImage *img = cvCreateImage(cvSize(width, height), src->depth, channels);
	cvCopy(src,img,NULL);

	data = (uchar *)img->imageData;
	// convolve the image
	forsn(i,1,height-1){ 
		forsn(j,1,width-1){
			forn(k,channels){
				acumX  = srcd[(i-1)*step+(j-1)*channels+k] 	; 	//m[0][0];
				acumX += srcd[(i)*step	+(j-1)*channels+k] 	;		//m[2][0];
				acumX += srcd[(i+1)*step+(j-1)*channels+k] 	;		//m[2][0];

				acumX -= srcd[(i-1)*step +(j+1)*channels+k]		;		//m[0][2];
				acumX -= srcd[i*step		+(j+1)*channels+k]	;		//m[1][2];
				acumX -= srcd[(i+1)*step	+(j+1)*channels+k]	;		//m[2][2];
			
				acumY  = srcd[(i-1)*step	+(j-1)*channels+k] 	;		//m[0][1];
				acumY += srcd[(i-1)*step	+(j)*channels+k]	;		//m[1][1];
				acumY += srcd[(i-1)*step	+(j+1)*channels+k] ;		//m[2][1];
				acumY -= srcd[(i+1)*step	+(j-1)*channels+k] 	;		//m[0][1];
				acumY -= srcd[(i+1)*step	+(j)*channels+k]	;		//m[1][1];
				acumY -= srcd[(i+1)*step	+(j+1)*channels+k]	;		//m[2][1];
				
				acum = sqrt(acumY*acumY + acumX*acumX);
				data[i*step+j*channels+k]= sat(acum);
			}
		}
	}
	return img;
}


int main( int argc, char** argv )
{
	time_t start,end;
	IplImage  *frame, *outFrame;
	int       key, counter ;
	/* supply the AVI file to play */
	assert( argc == 2 );

	/* load the AVI file */
	CvCapture *capture = cvCaptureFromAVI( argv[1] );

	/* always check */
	if( !capture ) return 1;    

	/* get fps, needed to set the delay */
	int fps = ( int )cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	/* display video */
	cvNamedWindow( "inputVideo", CV_WINDOW_AUTOSIZE );
	cvMoveWindow( "inputVideo", 10, 10);

	cvNamedWindow( "outputVideo", CV_WINDOW_AUTOSIZE );
	cvMoveWindow( "outputVideo", 1000, 10);

	time(&start);
	counter = 0;
	int m[9] = {-1,-1,-1,
				-1,8,-1,
				-1,-1,-1};

	while( key != 1048689){ //'q' ) {
		/* get a frame */
		frame = cvQueryFrame( capture );

		/* always check */
		if( !frame ) break;

		// get the image data
		/*height    = frame->height;
		width     = frame->width;
		step      = frame->widthStep;
		channels  = frame->nChannels;
		data      = (uchar *) frame->imageData;
		
		for(i=0;i<height;i++) for(j=0;j<width;j++) for(k=0;k<channels;k++)
			data[i*step+j*channels+k]= 200-data[i*step+j*channels+k] ;
*/
		
		outFrame = convolve_copy(frame, m);

		/* display frame */
		//cvShowImage( "inputVideo", frame );
		cvShowImage( "outputVideo", outFrame );

		cvReleaseImage(&outFrame);
		time(&end);
		++counter;
		double sec = difftime(end,start);
		double fpsRatio = counter/sec;
		cout << fpsRatio << "\t fps " << endl;
		/* quit if user press 'q' */
		key = cvWaitKey( 1000 / fps);
	}

	/* free memory */
	cvReleaseCapture( &capture );
	cvDestroyWindow( "video" );

	return 0;
}
